import { __assign, __awaiter, __generator } from "tslib";
import Response from './Response';
var Request = /** @class */ (function () {
    /**
     * Create a new api instance.
     */
    function Request(model) {
        /**
         * The default config.
         */
        this.config = {
            save: true
        };
        this.model = model;
        this.registerActions();
    }
    Object.defineProperty(Request.prototype, "axios", {
        /**
         * Get the axios client.
         */
        get: function () {
            if (!this.model.axios) {
                throw new Error('[Vuex ORM Axios] The axios instance is not registered. Please register the axios instance to the model.');
            }
            return this.model.axios;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Register actions from the model config.
     */
    Request.prototype.registerActions = function () {
        var actions = this.model.apiConfig.actions;
        if (!actions) {
            return;
        }
        for (var name_1 in actions) {
            var action = actions[name_1];
            typeof action === 'function'
                ? this.registerFunctionAction(name_1, action)
                : this.registerObjectAction(name_1, action);
        }
    };
    /**
     * Register the given object action.
     */
    Request.prototype.registerObjectAction = function (name, action) {
        var _this = this;
        this[name] = function (config) {
            return _this.request(__assign(__assign({}, action), config));
        };
    };
    /**
     * Register the given function action.
     */
    Request.prototype.registerFunctionAction = function (name, action) {
        this[name] = action.bind(this);
    };
    /**
     * Perform a get request.
     */
    Request.prototype.get = function (url, config) {
        if (config === void 0) { config = {}; }
        return this.request(__assign({ method: 'get', url: url }, config));
    };
    /**
     * Perform a post request.
     */
    Request.prototype.post = function (url, data, config) {
        if (data === void 0) { data = {}; }
        if (config === void 0) { config = {}; }
        return this.request(__assign({ method: 'post', url: url, data: data }, config));
    };
    /**
     * Perform a put request.
     */
    Request.prototype.put = function (url, data, config) {
        if (data === void 0) { data = {}; }
        if (config === void 0) { config = {}; }
        return this.request(__assign({ method: 'put', url: url, data: data }, config));
    };
    /**
     * Perform a patch request.
     */
    Request.prototype.patch = function (url, data, config) {
        if (data === void 0) { data = {}; }
        if (config === void 0) { config = {}; }
        return this.request(__assign({ method: 'patch', url: url, data: data }, config));
    };
    /**
     * Perform a delete request.
     */
    Request.prototype.delete = function (url, config) {
        if (config === void 0) { config = {}; }
        return this.request(__assign({ method: 'delete', url: url }, config));
    };
    /**
     * Perform an api request.
     */
    Request.prototype.request = function (config) {
        return __awaiter(this, void 0, void 0, function () {
            var requestConfig, response, entities;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        requestConfig = this.createConfig(config);
                        return [4 /*yield*/, this.axios.request(requestConfig)];
                    case 1:
                        response = _a.sent();
                        return [4 /*yield*/, this.persistResponseData(response, requestConfig)];
                    case 2:
                        entities = _a.sent();
                        return [2 /*return*/, new Response(this.model, requestConfig, response, entities)];
                }
            });
        });
    };
    /**
     * Create a new config by merging the global config, the model config,
     * and the given config.
     */
    Request.prototype.createConfig = function (config) {
        return __assign(__assign(__assign(__assign({}, this.config), this.model.globalApiConfig), this.model.apiConfig), config);
    };
    /**
     * Persist the response data to the vuex store.
     */
    Request.prototype.persistResponseData = function (response, config) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (!config.save) {
                    return [2 /*return*/, null];
                }
                if (config.delete !== undefined) {
                    return [2 /*return*/, this.model.delete(config.delete)];
                }
                return [2 /*return*/, this.model.insertOrUpdate({
                        data: this.getDataFromResponse(response, config)
                    })];
            });
        });
    };
    /**
     * Get data from the given response object. If the `dataKey` config is
     * provided, it tries to fetch the data at that key.
     */
    Request.prototype.getDataFromResponse = function (response, config) {
        if (!config.dataKey) {
            return response.data;
        }
        return response.data[config.dataKey];
    };
    return Request;
}());
export default Request;
//# sourceMappingURL=Request.js.map